Index: src/Fl_Pixmap.cxx
===================================================================
--- src/Fl_Pixmap.cxx	(revision 10751)
+++ src/Fl_Pixmap.cxx	(working copy)
@@ -186,22 +186,68 @@
   int X, Y, W, H;
   if (pxm->prepare(XP, YP, WP, HP, cx, cy, X, Y, W, H)) return;
   if (pxm->mask_) {
-    // I can't figure out how to combine a mask with existing region,
-    // so cut the image down to a clipped rectangle:
-    int nx, ny; fl_clip_box(X,Y,W,H,nx,ny,W,H);
-    cx += nx-X; X = nx;
-    cy += ny-Y; Y = ny;
-    // make X use the bitmap as a mask:
-    XSetClipMask(fl_display, fl_gc, pxm->mask_);
-    int ox = X-cx; if (ox < 0) ox += pxm->w();
-    int oy = Y-cy; if (oy < 0) oy += pxm->h();
-    XSetClipOrigin(fl_display, fl_gc, X-cx, Y-cy);
-  }
-  copy_offscreen(X, Y, W, H, pxm->id_, cx, cy);
-  if (pxm->mask_) {
+    // We can't directly combine a mask with an existing clip region, so we
+    // trim the mask using the current clip region by creating a new mask
+    // (bitmap) and copying the old mask into this bitmap with the existing
+    // clip region. We then use this new (clipped) mask to draw the pixmap.
+    //
+    // However, this is only necessary if the area we want to draw is not
+    // entirely inside the current clip region, i.e. the pixmap overlaps
+    // the clip region's borders.
+    //
+    // For more info see STR #3206.
+
+    // Get original clip region to trim the pixmap's mask and test
+    // if the area is entirely included in the clip region.
+    Region r = clip_region();
+    if (r && !(XRectInRegion(r, X, Y, W, H) == RectangleIn)) { // overlap
+
+      unsigned long black = BlackPixel(fl_display, DefaultScreen(fl_display));
+      unsigned long white = WhitePixel(fl_display, DefaultScreen(fl_display));
+
+      // create new mask bitmap (a Pixmap with depth 1)
+      Pixmap mask = XCreatePixmap(fl_display, RootWindow(fl_display, fl_screen), W, H, 1);
+
+      // create GC and fill mask with bg color (white = transparent)
+      GC gc = XCreateGC(fl_display, mask, 0, NULL);
+      XSetBackground(fl_display, gc, white);
+      XSetForeground(fl_display, gc, black);
+      XFillRectangle(fl_display, mask, gc, 0, 0, W, H);
+
+      // set original clip region on mask to trim it to the clip region
+      XSetRegion(fl_display, gc, r);
+      XSetClipOrigin(fl_display, gc, -X, -Y);
+
+      // copy and clip the original mask into the new mask bitmap
+      XSetBackground(fl_display, gc, black);
+      XSetForeground(fl_display, gc, white);
+      XCopyArea(fl_display, pxm->mask_, mask, gc, cx, cy, W, H, 0, 0);
+
+      // make X use the new bitmap as a mask:
+      XSetClipMask(fl_display, fl_gc, mask);
+      XSetClipOrigin(fl_display, fl_gc, X, Y);
+
+      // draw and clip the pixmap
+      copy_offscreen(X, Y, W, H, pxm->id_, cx, cy);
+
+      XFreeGC(fl_display, gc);
+      XFreePixmap(fl_display, mask);
+
+    } else { // no clip region or pixmap entirely inside the clip region
+
+      // make X use the pixmap's bitmap as a mask:
+      XSetClipMask(fl_display, fl_gc, pxm->mask_);
+      XSetClipOrigin(fl_display, fl_gc, X-cx, Y-cy);
+      copy_offscreen(X, Y, W, H, pxm->id_, cx, cy);
+
+    } // no clip region or inside ...
+
     // put the old clip region back
     XSetClipOrigin(fl_display, fl_gc, 0, 0);
     fl_restore_clip();
+
+  } else  { // no mask
+    copy_offscreen(X, Y, W, H, pxm->id_, cx, cy);
   }
 }
 
